@@ -35,7 +35,7 @@ no strict "subs";
 
 use vars qw/
 	$libname $rootnode $outputdir $opt $debug
-	$methodNumber
+	$methodNumber $headerSubdirectories
 	%builtins %typeunion %allMethods %allTypes %enumValueToType %typedeflist %mungedTypeMap
 	%skippedClasses /;
 
@@ -158,6 +158,8 @@ BEGIN
    'unsigned short int' => 'ushort',
 );
 
+$headerSubdirectories = "kio/|kdevelop/|kinterfacedesigner/|kontact/|kate/|kparts/|dom/|kabc/|ksettings/|kjs/|ktexteditor/|kdeprint/|kdesu/|knewstuff/"
+
 }
 
 sub writeDoc
@@ -180,7 +182,7 @@ sub writeDoc
 		kdocAstUtil::attachChild( $cNode, $node ) if ($cNode);
 		$node->AddProp( "Access", "public" );
 	}
-
+	
 	print STDERR "Preparsing...\n";
 
 	# Preparse everything, to prepare some additional data in the classes and methods
@@ -252,6 +254,7 @@ sub preParseClass
             $className eq 'QTSManip' || # cause compiler errors with several gcc versions
 	    $className eq 'QGDict' ||
 	    $className eq 'QGList' ||
+	    $className eq 'QGArray' ||
 	    $className eq 'QGVector' ||
 	    $className eq 'QStrIList' ||
 	    $className eq 'QStrIVec' ||
@@ -264,7 +267,20 @@ sub preParseClass
 	    ($className eq 'QSignal' and $main::qt_embedded) ||
 	    ($className eq 'QWSEvent' and $main::qt_embedded) ||
 	    ($className eq 'QMetaObjectInit' and $main::qt_embedded) ||
+	    ($className eq 'QKoi8Codec' and $main::qt_embedded) ||
 	    $className eq 'KAccelGen' ||
+	    ($className eq 'QAbstractUndoItem' and $main::qt4) ||
+	    ($className eq 'QDebug' and $main::qt4) ||
+	    ($className eq 'QNoDebug' and $main::qt4) ||
+	    ($className eq 'QObjectData' and $main::qt4) ||
+	    ($className eq 'QSysInfo' and $main::qt4) ||
+	    ($className eq 'QPNGImageWriter' and $main::qt4) ||
+	    ($className eq 'QPNGImagePacker' and $main::qt4) ||
+	    ($className eq 'QTextCodec::ConverterState' and $main::qt4) ||
+	    ($className eq 'QTextLayout::Selection' and $main::qt4) ||
+	    ($className eq 'QTextStreamManipulator' and $main::qt4) ||
+	    $className eq 'DCOPArg' ||
+	    $className eq 'DCOPReply' ||
 	    $className eq 'KBookmarkMenu::DynMenuInfo' ||
 	    $className eq 'KDE' ||
 	    $className eq 'KDEDesktopMimeType::Service' ||
@@ -278,9 +294,9 @@ sub preParseClass
 	    $className eq 'KEditListBox::CustomEditor' ||
 		$className eq 'KIO::KBookmarkMenuNSImporter' ||
 	    $className eq 'KExtendedSocket' ||
+	    $className eq 'KSocket' ||
 	    $className eq 'KPerDomainSettings' ||
 	    $className eq 'KApplicationPropsPlugin' ||
-	    $className eq 'KPrinter' ||
 	    $className eq 'KOpenWithHandler' ||
 	    $className eq 'KFileOpenWithHandler' ||
 	    $className eq 'KBindingPropsPlugin' ||
@@ -288,7 +304,6 @@ sub preParseClass
 	    $className eq 'KFileSharePropsPlugin' ||
 	    $className eq 'KBookmarkMenuNSImporter' ||
 	    $className eq 'KDevicePropsPlugin' ||
-	    $className eq 'KWin::WindowInfo' ||
 	    $className eq 'KDEDModule' ||
 	    $className eq 'KFileMetaInfoProvider' ||
 	    $className eq 'KFileMimeTypeInfo' ||
@@ -314,7 +329,6 @@ sub preParseClass
 	    $className eq 'khtml::MouseReleaseEvent' ||
 	    $className eq 'khtml::MouseEvent' ||
 	    $className eq 'khtml' ||
-	    $className eq 'KMimeType' ||
 	    $className eq 'KURL::List' ||
 	    $className eq 'KWin::Info' ||
 	    $className eq 'TerminalInterface' ||
@@ -345,6 +359,9 @@ sub preParseClass
 	my $hasCopyConstructor = 0;
 	my $hasPrivateCopyConstructor = 0;
 	# Note: no need for hasPureVirtuals. $classNode{Pure} has that.
+	
+	# Hack to fix up KLed constructors in KDE 3.1
+	my $kledAmbiguousConstructor = undef;
 
         my $doPrivate = $main::doPrivate;
 	$main::doPrivate = 1;
@@ -382,6 +399,7 @@ sub preParseClass
                         (my $classNameWithoutNS = $className) =~ s/^.*:://;
 			if ($theArgType =~ /$classNameWithoutNS\s*\&/) {
 			    $hasCopyConstructor = 1;
+				$m->{Flags} .= "x";
 			    $hasPrivateCopyConstructor = 1 if ( $m->{Access} eq 'private' );
 			}
 		    }
@@ -407,7 +425,10 @@ sub preParseClass
 		# or where the code won't compile/link for obscure reasons. Or even obvious reasons..
 		if ( ($classNode->{astNodeName} eq 'KCharSelectTable' and $name eq 'paintCell')
 			|| ($classNode->{astNodeName} eq 'KAnimWidget' and $name eq 'KAnimWidget' and @{$m->{ParamList}} == 2)
+			|| ($classNode->{astNodeName} eq 'KCModuleLoader' and $name eq 'errorModule')
 			|| ($classNode->{astNodeName} eq 'KDCOPActionProxy' and $name eq 'actions')
+			|| ($classNode->{astNodeName} eq 'KEditToolbarWidget' and $name eq 'insertActive')
+			|| ($classNode->{astNodeName} eq 'KEditToolbarWidget' and $name eq 'removeActive')
 			|| ($classNode->{astNodeName} eq 'KFileDialog' and $name eq 'addDirEntry')
 			|| ($classNode->{astNodeName} eq 'KFileDialog' and $name eq 'getDirEntry')
 			|| ($classNode->{astNodeName} eq 'KFileItem' and $name eq 'extraData')
@@ -437,6 +458,7 @@ sub preParseClass
 			|| ($classNode->{astNodeName} eq 'KEMailSettings' and $name eq 'getExtendedSetting')
 			|| ($classNode->{astNodeName} eq 'KEMailSettings' and $name eq 'setExtendedSetting')
 			|| ($classNode->{astNodeName} eq 'KProtocolManager' and $name eq 'defaultConnectTimeout')
+			|| ($classNode->{astNodeName} eq 'KMD4' and $name eq 'transform')
 			|| ($classNode->{astNodeName} eq 'KMD5' and $name eq 'transform')
 			|| ($classNode->{astNodeName} eq 'KSSLCertificate' and $name eq 'operator!=')
 			|| ($classNode->{astNodeName} eq 'KSSLPKCS7' and $name eq 'validate')
@@ -452,6 +474,12 @@ sub preParseClass
 			|| ($classNode->{astNodeName} eq 'KIO' and $name eq 'pasteData')
 			|| ($classNode->{astNodeName} eq 'KIO' and $name eq 'pasteDataAsync')
 			|| ($classNode->{astNodeName} eq 'KIO' and $name eq 'isClipboardEmpty')
+			|| ($classNode->{astNodeName} eq 'DCOPRef' and $name eq 'callExt')
+			|| ($classNode->{astNodeName} eq 'DCOPRef' and $name eq 'call')
+			|| ($classNode->{astNodeName} eq 'DCOPRef' and $name eq 'send')
+			|| ($classNode->{astNodeName} eq 'DOM' and $name eq 'operator<<') # Avoid kdbgstream debugging method
+			|| ($name eq 'qInitJpegIO' and $main::qt4)
+			|| ($name eq 'qInitPngIO' and $main::qt4)
 			|| ($name eq 'virtual_hook')
 			
 			# Various methods to skip in Qt/E (Qt 2.3.x)
@@ -492,6 +520,19 @@ sub preParseClass
 			next;
 		}
 
+		# Hack for fixing up KDE 3.1 KLed where the no arg constructor was ambiguous
+		if ($classNode->{astNodeName} eq 'KLed' and $name eq 'KLed' && $#{$m->{ParamList}} > 0) {
+			if ($m->{ParamList}[0]->{ArgType} =~ /QColor/ && defined $m->{ParamList}[0]->{DefaultValue}) {
+				$m->{ParamList}[0]->{DefaultValue} = undef;
+				if (defined $kledAmbiguousConstructor) {
+					$kledAmbiguousConstructor->{ParamList}[0]->{DefaultValue} = undef;
+					$kledAmbiguousConstructor->{FirstDefaultParam} = 1;
+				}
+			} else {
+				$kledAmbiguousConstructor = $m;
+			}
+		}
+		
 	    my $argId = 0;
 	    my $firstDefaultParam;
 	    foreach my $arg ( @{$m->{ParamList}} ) {
@@ -509,6 +550,7 @@ sub preParseClass
 		     or $arg->{ArgType} eq 'const QTextItem&' # ref to a private class in 3.2.0b1
 		     or $arg->{ArgType} eq 'FILE*' # won't be able to handle that I think
 		     or $arg->{ArgType} eq 'const KKeyNative&' #
+		     or $arg->{ArgType} =~ /Node\s*\*/ #
 		) {
 		    $m->{NodeType} = 'deleted';
 		}
@@ -643,7 +685,7 @@ sub propagateCanBeCopied($)
 	if ( !$classNode->{HasCopyConstructor} && $classNode->{CanBeCopied} && $classNode->{CanBeInstanciated} ) {
 	    my $methodNode = Ast::New( "$classNode->{astNodeName}" );
 	    $methodNode->AddProp( "NodeType", "method" );
-	    $methodNode->AddProp( "Flags", "" );
+	    $methodNode->AddProp( "Flags", "ix" ); # Only for internal use in marshallers
 	    my $argType = "const ".$className."&";
 	    registerType( $argType );
 	    $methodNode->AddProp( "Params", $argType );
@@ -684,9 +726,6 @@ my $fhn =1; # static
  	print $fh "#include <smoke.h>\n";
  	print $fh "#include <${libname}_smoke.h>\n";
 	
-	# Hack - some Qt headers need qregexp, but don't include it
- 	print $fh "#include <qregexp.h>\n";
-
         my @code;
         for my $node ( @$nodelist )
         {
@@ -695,6 +734,29 @@ my $fhn =1; # static
         my %includes;
         map { for my $incl (keys %{$_->[2]}){ $includes{$incl}++ } } @code;
 
+	# Hack - some Qt/KDE headers need other headers included, but omit suitable #includes
+	if (defined $includes{"qregexp.h"} || defined $includes{"qcstring.h"}) {
+	    print $fh "#include <qregexp.h>\n";
+		delete $includes{"qregexp.h"};
+	}
+	if (defined $includes{"qmime.h"}) {
+	    print $fh "#include <qurl.h>\n";
+		delete $includes{"qurl.h"};
+	}
+	if (defined $includes{"kshortcut.h"}) {
+	    print $fh "#include <kshortcut.h>\n";
+		delete $includes{"kshortcut.h"};
+	} 
+	if (defined $includes{"kshortcutlist.h"}) {
+	    print $fh "#include <kconfigbase.h>\n";
+	    print $fh "#include <kshortcutlist.h>\n";
+		delete $includes{"kconfigbase.h"};
+		delete $includes{"kshortcutlist.h"};
+	}
+	if (defined $includes{"kaction.h"}) {
+	    print $fh "#include <kaction.h>\n";
+		delete $includes{"kaction.h"};
+	}
 	foreach my $incl (keys %includes) {
 	    die if $incl eq '';
 	    print $fh "#include <$incl>\n";
@@ -891,7 +953,7 @@ sub addIncludeForClass($$$)
 {
     my ( $node, $addInclude, $debugMe ) = @_;
     my $sourcename = $node->{Source}->{astNodeName};
-    if ( $sourcename !~ s!.*(kio/|kparts/|dom/|kabc/|ksettings/|kjs/|ktexteditor/|kdeprint/|kdesu/)(.*)!$1$2!m ) {
+    if ( $sourcename !~ s!.*($headerSubdirectories)(.*)!$1$2!m ) {
     	$sourcename =~ s!.*/(.*)!$1!m;
 	}
 #    die "Empty source name for $node->{astNodeName}" if ( $sourcename eq '' );
@@ -1004,7 +1066,7 @@ sub generateVirtualMethod($$$$$)
 	    if($arg =~ s/&//) {
 		$cast = "*($arg *)";
 		$methodCode .= "${indent}return $cast$v;\n";
-	    } elsif($arg !~ /\*/) {
+	    } elsif($arg !~ /\*$/) {
 		unless($flags =~ "p") {
 		    $indent = "\t    ";
 		    $methodCode .= "{\n";
@@ -1260,15 +1322,18 @@ sub generateMethod($$$)
 }
 
 
-sub generateEnum($$)
+sub generateEnum($$$)
 {
-    my( $classNode, $m ) = @_;	# input
+    my( $classNode, $m, $addInclude ) = @_;	# input
     my $methodCode = '';	# output
     my $switchCode = '';	# output
 
     my @heritage = kdocAstUtil::heritage($classNode);
     my $className  = join( "::", @heritage );
     my $xClassName  = "x_" . join( "__", @heritage );
+    
+	my $fullEnumType = "$className\::". $m->{astNodeName};
+	checkIncludesForObject( $fullEnumType, $addInclude );
 
     foreach my $enum ( @{$m->{ParamList}} ) {
 	my $enumName = $enum->{ArgName};
@@ -1366,7 +1431,7 @@ sub generateAllMethods
     my $isGlobalSpace = ($xClassName eq ("x_".$main::globalSpaceClassName));
     my $sourcename = $classNode->{Source}->{astNodeName};
    
-    if ( $sourcename !~ s!.*(kio/|kparts/|dom/|kabc/|ksettings/|kjs/|ktexteditor/|kdeprint/|kdesu/)(.*)!$1$2!m ) {
+    if ( $sourcename !~ s!.*($headerSubdirectories)(.*)!$1$2!m ) {
     	$sourcename =~ s!.*/(.*)!$1!m;
 	}
     die "Empty source name for $classNode->{astNodeName}" if ( $sourcename eq '' );
@@ -1377,7 +1442,14 @@ sub generateAllMethods
         if($classNode->{NodeType} eq 'namespace') {
            $switchCode .= "    (void)obj;\n";
             $methodCode .= "public:\n";
-        } elsif(! $classNode->{BindingDerives}) {
+        	my $s;
+        	for my $sn( @{$classNode->{Sources}} ) {
+    		if ( ($s = $sn->{astNodeName}) !~ s!.*($headerSubdirectories)(.*)!$1$2!m ) {
+    			$s =~ s!.*/(.*)!$1!m;
+			}
+			$addInclude{ $s } = 1;
+        	}
+         } elsif(! $classNode->{BindingDerives}) {
             $methodCode .= "private:\n";
             $methodCode .= "    $className *xthis;\n";
             $methodCode .= "public:\n";
@@ -1390,7 +1462,7 @@ sub generateAllMethods
     } else {
         my $s;
         for my $sn( @{$classNode->{Sources}} ) {
-    	if ( ($s = $sn->{astNodeName}) !~ s!.*(kio/|kparts/|dom/|kabc/|ksettings/|kjs/|ktexteditor/|kdeprint/|kdesu/)(.*)!$1$2!m ) {
+    	if ( ($s = $sn->{astNodeName}) !~ s!.*($headerSubdirectories)(.*)!$1$2!m ) {
     		$s =~ s!.*/(.*)!$1!m;
 		}
 		$addInclude{ $s } = 1;
@@ -1405,7 +1477,7 @@ sub generateAllMethods
 			  sub {	my ($classNode, $methodNode ) = @_;
 				
 	if ( $methodNode->{NodeType} eq 'enum' ) {
-	    my ($meth, $swit) = generateEnum( $classNode, $methodNode );
+	    my ($meth, $swit) = generateEnum( $classNode, $methodNode, \%addInclude );
 	    $methodCode .= $meth;
 	    $switchCode .= $swit;
 	}
@@ -1580,6 +1652,9 @@ sub registerType($$) {
 
     $type =~ s/\s+const$//; # for 'char* const'
     $type =~ s/\s+const\s*\*$/\*/; # for 'char* const*'
+	
+	$type =~ s/(<[^>]*)\s+([^>]*>)/$1$2/; # Remove embedded space from template types, such as
+										  # 'QMap<QCString, DCOPRef>'
 
     return if ( $type eq 'void' or $type eq '' or $type eq '~' );
     die if ( $type eq '...' );     # ouch
@@ -1637,6 +1712,9 @@ sub findTypeEntry($) {
     $type =~ s/\s+const$//; # for 'char* const'
     $type =~ s/\s+const\s*\*$/\*/; # for 'char* const*'
 
+	$type =~ s/(<[^>]*)\s+([^>]*>)/$1$2/; # Remove embedded space from template types, such as
+										  # 'QMap<QCString, DCOPRef>'
+
     return undef if ( $type =~ '~' or $type eq 'void' or $type eq '' );
 
     # Enum _value_ -> get corresponding type
@@ -1996,7 +2074,10 @@ sub writeSmokeDataFile($) {
 	my $typeFlags = $allTypes{$type}{typeFlags};
 	my $realType = $allTypes{$type}{realType};
 	die "$type" if !defined $typeFlags;
-	die "$realType" if $realType =~ /\(/;
+#	die "$realType" if $realType =~ /\(/;
+	if ($realType =~ /\(/) {
+		print "FATAL ERROR $type $realType\n";
+	}
 	# First write the name
 	print OUT "\t{ \"$type\", ";
 	# Then write the classId (and find out the typeid at the same time)
@@ -2284,6 +2365,8 @@ sub writeSmokeDataFile($) {
 		# Make no distinction between a static method in an ordinary class, or a method in a namespace
 		$methodFlags .= "|Smoke::mf_static" if $m->{Flags} =~ "s" or $classNode->{NodeType} eq 'namespace';
 		$methodFlags .= "|Smoke::mf_const" if $m->{Flags} =~ "c"; # useful?? probably not
+		$methodFlags .= "|Smoke::mf_copyctor" if $m->{Flags} =~ "x";
+		$methodFlags .= "|Smoke::mf_internal" if $m->{Flags} =~ "i";
 		$methodFlags =~ s/0\|//; # beautify
 		
 		print OUT "\t{$classIndex, $methodIndex, $arglist, $argcnt, $methodFlags, $retTypeIndex, $case},\t//$methodCount $className\::$sig";
