@@ -9,7 +9,7 @@
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  but WITHOUT any WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Library General Public License for more details.
  *
@@ -28,6 +28,9 @@
 #include <kjs/object.h>
 #include <kjsembed/jsproxy.h>
 
+#include <algorithm>
+#include <typeinfo>
+
 class QTextStream;
 
 namespace KJSEmbed {
@@ -46,7 +49,12 @@ public:
     JSOpaqueProxy();
 
     /** Creates an opaque proxy. */
-    JSOpaqueProxy( void *ptr, const char *ptrtype );
+    template<typename T>
+    JSOpaqueProxy( T *nptr, const char *ptype )
+    : JSProxy( JSProxy::OpaqueProxy ), ptrtype(ptype), ptr(new Pointer<T>(nptr))
+    {
+
+    }
 
     /** Creates an opaque proxy. */
     JSOpaqueProxy( QTextStream *ts );
@@ -61,7 +69,15 @@ public:
     QString typeName() const;
 
     /** Sets the value of the proxy and its type. */
-    void setValue( void *ptr, const char *ptrtype );
+    template<typename T>
+    void setValue( T *nptr, const char *ptype ) {
+	  if( ptr ) {
+	    if(owner() == JavaScript) ptr->cleanup();
+	    delete ptr;
+	  }
+	  ptr = new Pointer<T>(nptr);
+	  ptrtype = ptype ? ptype : "void";
+    }
 
     /** Sets the value of the proxy to a QTextStream. */
     void setValue( QTextStream *ts );
@@ -69,8 +85,14 @@ public:
     /** Sets the value of the proxy to a QEvent. */
     void setValue( const QEvent *ev );
 
-    /** Returns the proxy value as a void pointer. */
-    void *toVoidStar() { return ptr; }
+    /** Returns the proxy value as a pointer. */
+    template<typename T>
+    T *toNative() {
+      if( !ptr ) return 0L; // empty
+      T *value = 0L;
+      return (T*)ptr->voidStar(); // Hack for now
+      return value;  // return real value
+    }
 
     /** Returns true iff the content of this proxy inherits the specified base-class. */
     bool inherits( const char *clazz );
@@ -90,12 +112,55 @@ public:
     virtual KJS::UString toString( KJS::ExecState *exec ) const;
 
 private:
-    void *ptr;
+
     QTextStream *textstream;
     const QEvent *event;
     QCString ptrtype;
 
     class JSOpaqueProxyPrivate *d;
+
+    // container objects
+    class PointerBase
+    {
+      public:
+	virtual ~PointerBase() {};
+	virtual void cleanup() = 0;
+	virtual const std::type_info &type() const = 0;
+	virtual void *voidStar() = 0;
+	//template<typename ValueType>
+	//virtual void castTo(ValueType *& val) = 0L;
+    };
+
+    template<typename ValueType>
+    class Pointer : public PointerBase
+    {
+      public:
+	Pointer( ValueType *value) : ptr(value) {}
+	~Pointer( ) {}
+	void cleanup()
+	{
+	  delete ptr;
+	  ptr=0L;
+	}
+	const std::type_info &type() const
+	{
+	  return typeid(ValueType);
+	}
+	
+	void *voidStar()
+	{
+	  return (void*)ptr;
+	}
+	//template<typename type>
+	//virtual void castTo(type *& val)
+	//{
+	//	val = dynamic_cast<type*>(ptr);
+	//	kdDebug() << "Type " << val << endl;
+	//}
+	ValueType *ptr;
+    };
+
+    PointerBase *ptr;
 };
 
 } // namespace KJSEmbed
