@@ -21,6 +21,7 @@
 
 #ifndef KJSEMBEDSLOTUTILS_H
 #define KJSEMBEDSLOTUTILS_H
+#include "global.h"
 
 #include <kjs/object.h>
 #include <kjsembed/jsbinding.h>
@@ -38,9 +39,9 @@ class JSObjectProxyImp;
  * Provides utility methods for slot invocation.
  *
  * @author Richard Moore, rich@kde.org
- * @version $Id: slotutils.h,v 1.28 2004/06/12 00:52:11 rich Exp $
+ * @version $Id: slotutils.h,v 1.31 2004/11/30 15:45:50 geiseri Exp $
  */
-class JSSlotUtils
+class KJSEMBED_EXPORT JSSlotUtils
 {
 public:
     /**
@@ -74,7 +75,7 @@ public:
 
 	SignatureIntIntIntIntBool, SignatureIntIntIntIntInt, SignatureIntIntIntIntIntInt,
         SignatureCustom
-	
+
     };
 
     /**
@@ -125,9 +126,9 @@ public:
     static void implantDateTime( KJS::ExecState *exec, QUObject *uo, const KJS::Value &v, QDateTime *datetime );
     /** Implant a QPixmap into a QUObject. */
     static void implantPixmap( KJS::ExecState *exec, QUObject *uo, const KJS::Value &v, QPixmap *pix );
-    
-    /** 
-     * Implant a value proxy into a QUObject. The type specified in the clazz parameter 
+
+    /**
+     * Implant a value proxy into a QUObject. The type specified in the clazz parameter
      * must match the type of the proxy'd value for the method to succeed.
      */
     static bool implantValueProxy( KJS::ExecState *exec, QUObject *uo,
@@ -135,8 +136,8 @@ public:
 
     /** Implant a QVariant into a QUObject. */
     static bool implantQVariant( KJS::ExecState *exec, QUObject *uo,const KJS::Value &v);
-    
-    /** 
+
+    /**
      * Implant an opaque proxy into a QUObject. The type specified in the
      * clazz parameter must match the type of the proxy'd value for the method
      * to succeed.
@@ -150,6 +151,12 @@ public:
      */
     static bool implantObjectProxy( KJS::ExecState *exec, QUObject *uo,
 				    const KJS::Value &v, const QString &clazz );
+    /** Implant a QString into a QUObject. */ 
+    static void implantStringList(KJS::ExecState *exec, QUObject *uo, 
+				  const KJS::Value &v, QStringList *lst );
+    
+    /** Implant a char* into a QUObject. */ 
+    static void implantCharStar( KJS::ExecState *exec, QUObject *uo, const KJS::Value &v );
 
 private:
     JSSlotUtils();
