@@ -51,6 +51,7 @@
 #define	SECT_IS_PRIVATE		0x0004	/* It is private. */
 #define	SECT_IS_SLOT		0x0008	/* It is a slot. */
 #define	SECT_IS_SIGNAL		0x0010	/* It is a signal. */
+#define	SECT_MASK		0x001f	/* The mask of all flags. */
 
 
 /* Handle class flags. */
@@ -62,6 +63,8 @@
 #define	CLASS_IS_OPAQUE		0x00100000	/* It is opaque. */
 #define	CLASS_HAS_VAR_HANDLERS	0x00200000	/* It has variable handlers. */
 #define	CLASS_DTOR_RELEASE_GIL	0x00400000	/* The dtor releases the GIL. */
+#define	CLASS_IS_PROTECTED	0x00800000	/* It is protected. */
+#define	CLASS_IS_PROTECTED_SAV	0x01000000	/* It is protected (saved). */
 
 #define	hasEnums(cd)		((cd) -> classflags & CLASS_HAS_ENUMS)
 #define	setHasEnums(cd)		((cd) -> classflags |= CLASS_HAS_ENUMS)
@@ -77,6 +80,12 @@
 #define	setIsOpaque(cd)		((cd) -> classflags |= CLASS_IS_OPAQUE)
 #define	hasVarHandlers(cd)	((cd) -> classflags & CLASS_HAS_VAR_HANDLERS)
 #define	setHasVarHandlers(cd)	((cd) -> classflags |= CLASS_HAS_VAR_HANDLERS)
+#define	isProtectedClass(cd)	((cd) -> classflags & CLASS_IS_PROTECTED)
+#define	setIsProtectedClass(cd)	((cd) -> classflags |= CLASS_IS_PROTECTED)
+#define	resetIsProtectedClass(cd)	((cd) -> classflags &= ~CLASS_IS_PROTECTED)
+#define	wasProtectedClass(cd)	((cd) -> classflags & CLASS_IS_PROTECTED_SAV)
+#define	setWasProtectedClass(cd)	((cd) -> classflags |= CLASS_IS_PROTECTED_SAV)
+#define	resetWasProtectedClass(cd)	((cd) -> classflags &= ~CLASS_IS_PROTECTED_SAV)
 #define	isReleaseGILDtor(c)	((cd) -> classflags & CLASS_DTOR_RELEASE_GIL)
 #define	setIsReleaseGILDtor(c)	((cd) -> classflags |= CLASS_DTOR_RELEASE_GIL)
 
@@ -103,6 +112,14 @@
 #define	setIsReleaseGILCtor(c)	((c) -> ctorflags |= CTOR_RELEASE_GIL)
 
 
+/* Handle member flags. */
+
+#define	MEMBR_NUMERIC		0x0001		/* It is a numeric slot. */
+
+#define	isNumeric(m)		((m) -> memberflags & MEMBR_NUMERIC)
+#define	setIsNumeric(m)		((m) -> memberflags |= MEMBR_NUMERIC)
+
+
 /* Handle enum flags. */
 
 #define	isProtectedEnum(e)	((e) -> pcd != NULL)
@@ -235,6 +252,7 @@
 
 #define	isDuplicateVH(vh)	((vh) -> vhflags & VH_IS_DUPLICATE)
 #define	setIsDuplicateVH(vh)	((vh) -> vhflags |= VH_IS_DUPLICATE)
+#define	resetIsDuplicateVH(vh)	((vh) -> vhflags &= ~VH_IS_DUPLICATE)
 #define	isTransferVH(vh)	((vh) -> vhflags & VH_TRANSFERS)
 #define	setIsTransferVH(vh)	((vh) -> vhflags |= VH_TRANSFERS)
 
@@ -290,7 +308,10 @@ typedef enum {
 } slotType;
 
 
-/* Argument types. */
+/*
+ * Argument types.  Always add new ones at the end because the numeric values
+ * can appear in generated code.
+ */
 
 typedef enum {
 	no_type,
@@ -327,7 +348,8 @@ typedef enum {
 	pydict_type,
 	pycallable_type,
 	pyslice_type,
-	qobject_type
+	qobject_type,
+	function_type
 } argType;
 
 
@@ -495,7 +517,7 @@ typedef struct {
 	int			nrderefs;	/* Nr. of dereferences. */
 	valueDef		*defval;	/* The default value. */
 	union {
-		struct _signatureDef	*sa;	/* If it is a slot. */
+		struct _signatureDef	*sa;	/* If it is a function. */
 		struct _templateDef	*td;	/* If it is a template. */
 		struct _scopedNameDef	*snd;	/* If it is a defined type. */
 		struct _classDef	*cd;	/* If it is a class. */
@@ -657,6 +679,7 @@ typedef struct _enumDef {
 
 typedef struct _memberDef {
 	nameDef			*pyname;	/* The Python name. */
+	int			memberflags;	/* The member flags. */
 	slotType		slot;		/* The slot type. */
 	moduleDef		*module;	/* The owning module. */
 	struct _memberDef	*next;		/* Next in the list. */
@@ -683,8 +706,8 @@ typedef struct _classList {
 /* A virtual overload definition. */
 
 typedef struct _virtOverDef {
-	overDef			*o;		/* The overload. */
-	struct _classDef	*nearc;		/* The nearest class. */
+	overDef			o;		/* The overload. */
+	struct _classDef	*scope;		/* The overload scope. */
 	struct _virtOverDef	*next;		/* Next in the list. */
 } virtOverDef;
 
